@@ -4,7 +4,7 @@ methods that act upon a transfer function to gain functions such as the
 mass function.
 '''
 
-version = '1.1.0'
+version = '1.1.1'
 
 ###############################################################################
 # Some Imports
@@ -32,11 +32,31 @@ class Perturbations(object):
     methods of the class need no further input after the cosmology has been initialized.
     
     Contains an update() method which can be passed arguments to update, in the most optimal manner.
+    All output quantities are called as properties of the class, and are calculated at need (but
+    stored after first calculation for quick access).
     
-    Required Input:
-        M: a float or vector of floats containing the log10(Solar Masses) at which to perform analysis.
+    Output Quantities:
+        sigma:         The mass variance of spheres of the given radius/mass. NOTE: not sigma^2
+        lnsigma:       The natural logarithm of the inverse of sigma
+        lnk:           The natural log of the wavenumbers in the power spectrum [h/Mpc]
+        growth:        The growth factor for the given cosmology and redshift
+        power:         The natural log of the normalised power spectrum for the given cosmology (at lnk)
+        n_eff:         Effective spectral index at the radius of a halo
+        M:             The masses at which analysis is performed. (not log) [M/h]
+        fsigma:        The multiplicity function, or fitting function at M
+        dndm:          The comoving number density of halos in mass interval M [h**3/Mpc**3]
+        dndlnm:        The comoving number density of halos in log mass interval M [h**3/Mpc**3]
+        dndlog10m:     The comoving number density of halo in log10 mass interval M [h**3/Mpc**3]
+        ngtm:          Comoving number density of halos > M [h**3/Mpc**3]
+        nltm:          Comoving number density of halos < M [h**3/Mpc**3]
+        mgtm:          Comoving mass density of halos > M [h**3/Mpc**3]
+        mltm:          Comoving mass density of halos < M [h**3/Mpc**3]
+        how_big:       The requisite size of a simulation box, L, to have at least one halo > M [Mpc/h]
         
-    Optional Input:
+    Input:
+        M:             A vector of floats containing the log10(Solar Masses) at which to perform analysis.    
+                       Default: M = np.linspace(10,15,501)
+                       
         transfer_file: Either a string pointing to a file with a CAMB-produced transfer function,
                        or None. If None, will use CAMB on the fly to produce the function.
                        Default is None.
@@ -44,16 +64,42 @@ class Perturbations(object):
         z:             a float giving the redshift of the analysis.
                        Default z = 0.0
                    
-        WDM:           a float giving warm dark matter particle size in keV. 
-                       Default is None (corresponds to CDM)
+        wdm_mass:      a float giving warm dark matter particle size in keV, or None for CDM. 
+                       Default is wdm_mass = None
                                                        
         k_bounds:      a list/tuple defining two values: the lower and upper limit of k. Used to truncate/extend
                        the power spectrum. 
-                       Default [0.0000001,20000.0]
-   
-        extrapolate:   Whether to use the k_bounds for extrapolation/truncation
-                       Default: True
+                       Default k_bounds = [0.0000001,20000.0]
                      
+        mf_fit:        A string indicating which fitting function to use for f(sigma)
+                       Default: mf_fit = 'ST'
+                       
+                       Options:                               
+                            1. 'PS': Press-Schechter Approach
+                            2. 'ST': Sheth-Tormen
+                            3. 'Jenkins': Jenkins empirical fit
+                            4. 'Warren': Warren empirical fit
+                            5. 'Reed03': Reed empirical from 2003
+                            6. 'Reed07': Reed empirical from 2007
+                            7. 'Tinker': Tinker empirical from 2008
+                            8. 'Watson': Watson empirical 2012
+                            9. 'Watson_FoF': Watson Friend-of-friend fit 2012
+                            10. 'Crocce': Crocce 2010
+                            11. 'Courtin': Courtin 2011
+                            12. 'Angulo': Angulo 2012
+                            13. 'Angulo_Bound': Angulo sub-halo function 2012
+                            14. 'user_model': A user-input string function
+        
+        delta_vir:     The virial overdensity for the halo definition
+                       Default: delta_vir = 200.0
+                       
+        user_fit:      A string defining a mathematical function in terms of 'x', used as the fitting function,
+                       where x is taken as sigma. Will only be applicable if mf_fit == "user_model".
+                       Default: user_fit = ""
+                       
+        transfer_fit:  A string defining which transfer function fit to use. Current options are 'CAMB' and 'EH' (Eistenstein-Hu)
+                       Default: transfer_fit = "CAMB"
+                       
         **kwargs:      There is a placeholder for any additional cosmological parameters, or camb
                        parameters, that one wishes to include. Parameters that aren't used won't
                        break the program, they will just be ignored. Here follows a list of parameters
@@ -99,30 +145,16 @@ class Perturbations(object):
                        DoLensing       :: False 
                        ThreadNum       :: 0 
                                 
-                                
-                        1. 'PS': Press-Schechter Approach
-                        2. 'ST': Sheth-Tormen
-                        3. 'Jenkins': Jenkins empirical fit
-                        4. 'Warren': Warren empirical fit
-                        5. 'Reed03': Reed empirical from 2003
-                        6. 'Reed07': Reed empirical from 2007
-                        7. 'Tinker': Tinker empirical from 2008
-                        8. 'Watson': Watson empirical 2012
-                        9. 'Watson_FoF': Watson Friend-of-friend fit 2012
-                        10. 'Crocce': Crocce 2010
-                        11. 'Courtin': Courtin 2011
-                        12. 'Angulo': Angulo 2012
-                        13. 'Angulo_Bound': Angulo sub-halo function 2012
-                        14. 'user_model': A user-input string function
+
     """
 
 
-    def __init__(self, M,
+    def __init__(self, M=np.linspace(10, 15, 501),
                  mf_fit="ST",
                  transfer_file=None,
                  z=0.0,
                  wdm_mass=None, k_bounds=[0.0000001, 20000.0],
-                 delta_vir=200.0, user_fit='', **kwargs):
+                 delta_vir=200.0, user_fit='', transfer_fit='CAMB', ** kwargs):
         """
         Initializes the cosmology for which to perform the perturbation analysis.      
         """
@@ -169,12 +201,6 @@ class Perturbations(object):
                                  'w_perturb'      : False,
                                  'DoLensing'       : False }
 
-#        self.run_params = {'M':M,
-#                           'transfer_file':transfer_file,
-#                           'z':z,
-#                           'wdm_mass':wdm_mass,
-#                           'k_bounds':k_bounds
-#                           }
 
         #A dictionary of available HMF fitting functions and their identifiers
         self.mf_fits = {
@@ -194,50 +220,9 @@ class Perturbations(object):
             "Bhattacharya": self._nufnu_Bhattacharya,
             "user_model":self._nufnu_user_model
             }
-#        #Cosmo parameters for CAMB
-#        self.__w_lam = -1
-#        self.__omegab = 0.0455
-#        self.__omegac = 0.226
-#        self.__omegav = 0.728
-#        self.__omegan = 0.0
-#        self.__H0 = 70.4
-#        self.__cs2_lam = 0
-#        self.__TCMB = 2.725
-#        self.__yhe = 0.24
-#        self.__Num_Nu_massless = 3.04
-#        self.__reion__optical_depth = 0.085
-#        self.__reion__redshift = 10.3
-#        self.__omegak = 1 - self.__omegab - self.__omegav - self.__omegac - self.__omegan
-#
-#        #Other cosmological parameters
-#        self.__sigma_8 = 0.81
-#        self.__n = 0.967
-#        self.__delta_c = 1.686
-#        self.__crit_dens = 27.755 * 10 ** 10
-#        self.__mean_dens = (self.__omegac + self.__omegab) * 27.755 * 10 ** 10
-#
-#        #CAMB options
-#        self.__Num_Nu_massive = 0
-#        self.__reion__fraction = -1
-#        self.__reion__delta_redshift = 1.5
-#        self.__Scalar_initial_condition = 1
-#        self.__scalar_amp = 1
-#        self.__scalar_running = 0
-#        self.__tensor_index = 0
-#        self.__tensor_ratio = 1
-#        self.__lAccuracyBoost = 1
-#        self.__lSampleBoost = 1
-#        self.__AccuracyBoost = 1
-#        self.__WantScalars = True
-#        self.__WantTensors = False
-#        self.__reion__reionization = True
-#        self.__reion__use_optical_depth = True
-#        self.__w_perturb = False
-#        self.__DoLensing = False
 
-        # "Run" parameters
         self.update(M=M, mf_fit=mf_fit, k_bounds=k_bounds, transfer_file=transfer_file, wdm_mass=wdm_mass,
-                     delta_vir=delta_vir, user_fit=user_fit, z=z, ** kwargs)
+                     delta_vir=delta_vir, user_fit=user_fit, z=z, transfer_fit=transfer_fit, ** kwargs)
 
     def update(self, **kwargs):
         """
@@ -299,7 +284,9 @@ class Perturbations(object):
                 self.delta_vir = kwargs['delta_vir']
             elif key is "user_fit":
                 self.user_fit = kwargs['user_fit']
-
+            elif key is "transfer_fit":
+                print "updating transfer_fit"
+                self.transfer_fit = kwargs['transfer_fit']
 
             else:
                 print "WARNING: ", key, " is not a valid parameter for the Perturbations class"
@@ -311,9 +298,6 @@ class Perturbations(object):
             if kwargs['z'] == 0:
                 del self.growth
 
-    @property
-    def dlogM(self):
-        return self.M[1] - self.M[0]
 
     @property
     def cosmo_params(self):
@@ -534,7 +518,7 @@ class Perturbations(object):
         try:
             open(val)
         except IOError:
-            raise IOError("The file ", val, " does not exist or cannot be opened")
+            raise IOError("The file " + val + " does not exist or cannot be opened")
 
         #Here we delete properties that are directly dependent on transfer_file
         del self._lnk_original
@@ -555,20 +539,42 @@ class Perturbations(object):
         del self.fsigma
 
     @property
-    def _lnk_original(self):
+    def transfer_fit(self):
+        """
+        A string specifying a method with which to calculate the transfer function.
+        
+        Currenty implemented are 'CAMB' (Code for anisotropies in the Microwave Background) and 'EH' (Eisenstein-Hu fit)
+        """
+        return self.__transfer_fit
+
+    @transfer_fit.setter
+    def transfer_fit(self, val):
+        if val not in ["CAMB", "EH"]:
+            raise ValueError("Sorry the transfer_fit has not been implemented for " + val + ". Please choose 'CAMB' or 'EH'")
+
+        self.__transfer_fit = val
+
+    #--------------------------------  START NON-SET PROPERTIES ----------------------------------------------
+    @property
+    def dlogM(self):
+        return self.M[1] - self.M[0]
+
+    @property
+    def _transfer_original(self):
         """
         Original values of lnk from the call to the transfer function
         """
         try:
-            return self.__lnk_original
+            return self.__transfer_original
         except:
-            self.__lnk_original, self.__transfer_original = tools.get_transfer(self._transfer_file, self.camb_params)
-            return self.__lnk_original
+            self.__transfer_original = tools.get_transfer(self._transfer_file, self.camb_params,
+                                                          self.transfer_fit, self.k_bounds)
+            return self.__transfer_original
 
-    @_lnk_original.deleter
-    def _lnk_original(self):
+    @_transfer_original.deleter
+    def _transfer_original(self):
         try:
-            del self.__lnk_original
+            del self.__transfer_original
             del self._transfer_function_callable
         except:
             pass
@@ -581,8 +587,8 @@ class Perturbations(object):
         try:
             return self.__transfer_function_callable
         except:
-            lnk_orig = self._lnk_original
-            self.__transfer_function_callable = tools.interpolate_transfer(lnk_orig, self.__transfer_original)
+            self.__transfer_function_callable = tools.interpolate_transfer(self._transfer_original[0, :],
+                                                                           self._transfer_original[1, :])
             return self.__transfer_function_callable
 
     @_transfer_function_callable.deleter
@@ -592,7 +598,6 @@ class Perturbations(object):
 
             #Also delete things directly dependent on it (or other self variables defined in it)
             del self.lnk
-            del self._transfer_function_callable
         except:
             pass
 
@@ -604,7 +609,11 @@ class Perturbations(object):
         try:
             return self.__lnk
         except:
-            self.__lnk, dlnk = tools.new_k_grid(self._lnk_original, self.k_bounds)
+            if self.transfer_fit == "CAMB":
+                self.__lnk, dlnk = tools.new_k_grid(self._transfer_original[0, :], self.k_bounds)
+            elif self.transfer_fit == "EH":
+                self.__lnk = self._transfer_original[0, :]
+
             # CHECK KR_BOUNDS
             self.max_error, self.min_error = tools.check_kr(self.M[0], self.M[-1], self.cosmo_params['mean_dens'],
                                                             np.exp(self.__lnk[0]), np.exp(self.__lnk[-1]))
@@ -639,7 +648,10 @@ class Perturbations(object):
         try:
             return self.__unnormalized_power
         except:
-            self.__unnormalized_power = self.cosmo_params['n'] * self.lnk + 2.0 * self._transfer_function_callable(self.lnk)
+            if self.transfer_fit == "CAMB":
+                self.__unnormalized_power = self.cosmo_params['n'] * self.lnk + 2.0 * self._transfer_function_callable(self.lnk)
+            else:
+                self.__unnormalized_power = self.cosmo_params['n'] * self.lnk + 2.0 * self._transfer_original[1, :]
             return self.__unnormalized_power
     @_unnormalized_power.deleter
     def _unnormalized_power(self):
@@ -679,6 +691,7 @@ class Perturbations(object):
             return self.__power_0
         except:
             if self.wdm_mass is not None:
+                print "Doing wdm with mass = ", self.wdm_mass
                 self.__power_0 = tools.wdm_transfer(self.wdm_mass, self._power_cdm_0, self.lnk,
                                                     self.cosmo_params["H0"], self.cosmo_params['omegac'])
             else:
@@ -1072,7 +1085,7 @@ class Perturbations(object):
         Calculates how big a box must be to expect one halo in natural log mass interval M, for given mass function
         """
 
-        return self.dndm ** (-1. / 3.)
+        return self.ngtm ** (-1. / 3.)
 
     @how_big.deleter
     def how_big(self):
@@ -1256,7 +1269,7 @@ class Perturbations(object):
         return vfv
 
     def _watson_gamma(self):
-        C = 0.947 * np.exp(0.023 * (self.delta_vir / 178 - 1))
+        C = np.exp(0.023 * (self.delta_vir / 178 - 1))
         d = -0.456 * cosmography.omegam_z(self.z, self.cosmo_params['omegam'], self.cosmo_params['omegav'], self.cosmo_params['omegak']) - 0.139
         p = 0.072
         q = 2.13
@@ -1265,7 +1278,7 @@ class Perturbations(object):
 
 
     def _nufnu_Watson_FoF(self):
-        vfv = 0.282 * ((2.163 / self.sigma) ** 1.406 + 1) * np.exp(-1.21 / self.sigma ** 2)
+        vfv = 0.282 * ((1.406 / self.sigma) ** 2.163 + 1) * np.exp(-1.21 / self.sigma ** 2)
         vfv[np.logical_or(self.lnsigma < -0.55 , self.lnsigma > 1.31)] = np.NaN
         return vfv
 
